import 'package:dart_aas/dart_aas.dart';
import 'package:s3i_flutter/src/broker/messages/i40_message.dart';
import 'package:test/test.dart';

void mainMessageTests() {
  test('Test I40InvokeOperation Sync/Async Request/Reply', () {
    final List<SubmodelElement> inputList = <SubmodelElement>[
      Property<int>(idShort: 'a', value: 1),
      Property<bool>(idShort: 'b', value: true),
      Property<String>(idShort: 'c', value: 'Hallo'),
      //TODO: is this working?
      Property<SubmodelElementCollection>(
          idShort: 'pose',
          value: SubmodelElementCollection(
            idShort: 'pose',
          )..elements = <SubmodelElement>[Property<int>(idShort: 'x', value: 1), Property<int>(idShort: 'y', value: 2)])
    ];
    final I40InvokeOperationAsyncRequest requestMessage = I40InvokeOperationAsyncRequest(
      submodelId: 'mySubmodel',
      operationIdShortPath: 'myOperation',
      sender: 'mySender',
      receiver: 'myReceiver',
      inputVariables: inputList,
    );

    var parsedRequestMessage = I40Message.fromJson(requestMessage.toJson());
    expect(parsedRequestMessage.runtimeType, I40InvokeOperationAsyncRequest);
    if (parsedRequestMessage is I40InvokeOperationAsyncRequest) {
      expect(parsedRequestMessage.submodelId, 'mySubmodel');
      expect(parsedRequestMessage.operationIdShortPath, 'myOperation');
      expect(parsedRequestMessage.sender, 'mySender');
      expect(parsedRequestMessage.receiver, 'myReceiver');
      //TODO
      // expect(parsedRequestMessage.inputMap['a'], 1);
      // expect(parsedRequestMessage.inputMap['b'], true);
      // expect(parsedRequestMessage.inputMap['c'], 'Hallo');
      // expect(parsedRequestMessage.inputMap['d'], {'x': 1, 'y': 2});
    }

    final List<SubmodelElement> outputList = <SubmodelElement>[
      Property<int>(idShort: 'a', value: 1),
      Property<bool>(idShort: 'b', value: true),
      Property<String>(idShort: 'c', value: 'Hallo'),
      //TODO: is this working?
      Property<SubmodelElementCollection>(
          idShort: 'pose',
          value: SubmodelElementCollection(
            idShort: 'pose',
          )..elements = <SubmodelElement>[Property<int>(idShort: 'x', value: 1), Property<int>(idShort: 'y', value: 2)])
    ];
    final I40InvokeOperationAsyncReply replyMsg = I40InvokeOperationAsyncReply(
        sender: 'mySender',
        receiver: 'myReceiver',
        outputVariables: outputList,
        success: false,
        errorMessage: 'Falscher Parameter',
        statusCode: 'KeyError');
    final I40Message parsedReplyMessage = I40Message.fromJson(replyMsg.toJson());
    if (parsedReplyMessage is I40InvokeOperationAsyncReply) {
      expect(parsedReplyMessage.errorMessage, 'Falscher Parameter');
      expect(parsedReplyMessage.statusCode, 'KeyError');
      expect(parsedReplyMessage.sender, 'mySender');
      expect(parsedReplyMessage.receiver, 'myReceiver');
    }
  });

  test('Test I40GetSubmodelElementByPath Request/Reply', () {
    final I40GetSubmodelElementByPathRequest requestMessage = I40GetSubmodelElementByPathRequest(
        submodelId: 'urn:s3i:1234.mySubmodel',
        submodelElementIdShortPath: 'myElement',
        sender: 'sender',
        receiver: 'receiver');

    var parsedRequestMessage = I40Message.fromJson(requestMessage.toJson());
    expect(parsedRequestMessage.runtimeType, I40GetSubmodelElementByPathRequest);
    parsedRequestMessage = parsedRequestMessage as I40GetSubmodelElementByPathRequest;
    expect(parsedRequestMessage.submodelId, 'urn:s3i:1234.mySubmodel');
    expect(parsedRequestMessage.submodelElementIdShortPath, 'myElement');
    expect(parsedRequestMessage.sender, 'sender');
    expect(parsedRequestMessage.receiver, 'receiver');

    final I40GetSubmodelElementByPathReply replyMessage = I40GetSubmodelElementByPathReply(
        sender: 'sender',
        receiver: 'receiver',
        submodelElement: Property<String>(
            idShort: 'myElement',
            semanticId: Reference(
                type: ReferenceTypes.ExternalReference,
                keys: [ReferenceKey(type: ReferenceKeyTypes.GlobalReference, value: "mySemanticReference")]),
            value: 'myValue'));

    var parsedReplyMessage = I40Message.fromJson(replyMessage.toJson());
    expect(parsedReplyMessage.runtimeType, I40GetSubmodelElementByPathReply);
    parsedReplyMessage = parsedReplyMessage as I40GetSubmodelElementByPathReply;
    expect(parsedReplyMessage.sender, 'sender');
    expect(parsedReplyMessage.receiver, 'receiver');
    expect(parsedReplyMessage.submodelElement.runtimeType, Property<String>);
    parsedReplyMessage.submodelElement = parsedReplyMessage.submodelElement as Property<String>;
    expect(parsedReplyMessage.submodelElement.idShort, 'myElement');
    expect((parsedReplyMessage.submodelElement as Property).value, 'myValue');
  });
}

void main() {
  mainMessageTests();
}
